@@ -24,6 +24,7 @@ from typing import Any, Generic, TypeVar
 import warnings
 
 from flax import traceback_util
+from flax import traverse_util
 from flax.typing import (
     In,
     InOutAxis,
@@ -127,7 +128,7 @@ def _partial_pack(
     out_variable_filters: Sequence[CollectionFilter],
     rng_filters: Sequence[PRNGSequenceFilter],
     name=None,
-) -> tuple[Callable[..., Any], Callable[..., Any], Any, Any, Callable[..., Any], Callable[..., Any]]:
+) -> tuple[Callable[..., Any], Callable[..., Any], Any, Any, Callable[..., Any]]:
   """Pack variables and rngs for functional transformations.
 
   The _partial_pack function is the building block for all other lifted transformations.
@@ -175,16 +176,11 @@ def _partial_pack(
     inner_rng_counters.append(rng_counters)
   rng_groups_xs_t = _transpose(rng_groups_xs)
 
-  inner_scopes: list[Scope] = []
-
   def scope_fn(
       variable_groups_xs_t,
       rng_groups_xs_t,
       mutable_filter: CollectionFilter = True,
   ):
-    nonlocal inner_scopes
-    for inner_scope in inner_scopes:
-      inner_scope.invalidate()
     inner_scopes = []
     mutable: Filter = False
     for out_filter in out_variable_filters:
@@ -260,10 +256,6 @@ def _partial_pack(
 
     return _transpose(out_variable_groups_xs)
 
-  def invalidate_scopes_fn():
-    for inner_scope in inner_scopes:
-      inner_scope.invalidate()
-
   def publish_results_fn(out_variable_groups_xs_t):
     out_variable_groups_xs = _transpose(out_variable_groups_xs_t)
     for scope, out_variable_groups, rng_counters in zip(
@@ -278,14 +270,14 @@ def _partial_pack(
             scope.put_variable(col_name, var_name, value)
 
   return (
-        scope_fn,
-        repack_fn,
-        variable_groups_xs_t,
-        rng_groups_xs_t,
-        publish_results_fn,
-        invalidate_scopes_fn,
+      scope_fn,
+      repack_fn,
+      variable_groups_xs_t,
+      rng_groups_xs_t,
+      publish_results_fn,
     )
 
+
 def pack(
     fn: Callable[..., Any],
     in_variable_filters: Sequence[CollectionFilter],
@@ -322,24 +314,20 @@ def pack(
         variable_groups_xs_t,
         rng_groups_xs_t,
         publish_results_fn,
-        invalidate_scopes_fn,
     ) = _partial_pack(scope_tree, in_variable_filters, out_variable_filters, rng_filters, name)
-    try:
-      if enable_kwargs:
-        y, out_variable_groups_xs_t = fn(
-            scope_fn,
-            repack_fn,
-            variable_groups_xs_t,
-            rng_groups_xs_t,
-            *args,
-            **kwargs,
-        )
-      else:
-        y, out_variable_groups_xs_t = fn(
-            scope_fn, repack_fn, variable_groups_xs_t, rng_groups_xs_t, *args
-        )
-    finally:
-      invalidate_scopes_fn()
+    if enable_kwargs:
+      y, out_variable_groups_xs_t = fn(
+          scope_fn,
+          repack_fn,
+          variable_groups_xs_t,
+          rng_groups_xs_t,
+          *args,
+          **kwargs,
+      )
+    else:
+      y, out_variable_groups_xs_t = fn(
+          scope_fn, repack_fn, variable_groups_xs_t, rng_groups_xs_t, *args
+      )
     publish_results_fn(out_variable_groups_xs_t)
     return y
 
@@ -891,6 +879,7 @@ def scan(
   _split_transpose: bool = False,
   data_transform: Callable[..., Any] | None = None,
   metadata_params: dict[Any, Any] = {},
+  check_constancy_invariants: bool = True,
 ) -> Callable[..., Any]:
   """A lifted version of ``jax.lax.scan``.
 
@@ -958,6 +947,11 @@ def scan(
       intended for inline SPMD annotations.
     metadata_params: arguments dict passed to AxisMetadata instances in the
       variable tree.
+    check_constancy_invariants: If true, the scan will verify that the
+      broadcast constants are true loop invariants, and further supports
+      broadcast function (non-carry) outputs.  This requires an extra jax
+      tracing step however, so setting to false can reduce trace time on larger
+      models.
 
   Returns:
     The scan function with the signature
@@ -1012,7 +1006,8 @@ def scan(
         length=length,
         reverse=reverse,
         unroll=unroll,
-        _split_transpose=_split_transpose
+        _split_transpose=_split_transpose,
+        check_constancy_invariants=check_constancy_invariants,
     )
     def scanned(broadcast_vars, carry, scan_variable_groups, rng_groups, args):
       carry_vars, c = carry
@@ -1512,6 +1507,81 @@ def _hashable_filter(x):
   return x
 
 
+class CountsHolder:
+
+  def __init__(self, flat_d):
+    self.flat_d = flat_d
+
+  @classmethod
+  def make(cls, d):
+    flat_d = traverse_util.flatten_dict(d)
+    flat_d = {k: v for k, v in flat_d.items()}
+    return cls(flat_d)
+
+  def sub(self, other):
+    delta_flat_d = {}
+    new_flat_d = collections.defaultdict(int, self.flat_d)
+    old_flat_d = collections.defaultdict(int, other.flat_d)
+    for k in new_flat_d:
+      delta_flat_d[k] = new_flat_d[k] - old_flat_d[k]
+    return CountsHolder(delta_flat_d)
+
+  def add(self, other):
+    delta_flat_d = {}
+    new_flat_d = collections.defaultdict(int, self.flat_d)
+    old_flat_d = collections.defaultdict(int, other.flat_d)
+    for k in new_flat_d:
+      delta_flat_d[k] = new_flat_d[k] + old_flat_d[k]
+    return CountsHolder(delta_flat_d)
+
+  def unflat(self):
+    return traverse_util.unflatten_dict(self.flat_d)
+
+
+def set_from_dict(original, updates):
+  for k in updates:
+    if k not in original:
+      original[k] = updates[k]
+    else:
+      if isinstance(updates[k], dict):
+        set_from_dict(original[k], updates[k])
+      else:
+        original[k] = updates[k]
+
+
+class _SideEffectCache(threading.local):
+
+  def __init__(self):
+    self.cache = {}
+
+
+_side_effect_cache = _SideEffectCache()
+
+
+def _restore_rng_counters(scopes, fingerprint, capture_old_counts):
+  if fingerprint not in _side_effect_cache.cache:
+    capture_new_counts = jax.tree.map(
+        lambda s: CountsHolder.make(s.rng_counters), scopes
+    )
+    capture_delta_counts = jax.tree.map(
+        lambda old, new: new.sub(old),
+        capture_old_counts,
+        capture_new_counts,
+    )
+    _side_effect_cache.cache[fingerprint] = capture_delta_counts
+  else:
+    updated_counts = jax.tree.map(
+        lambda x, y: x.add(y).unflat(),
+        _side_effect_cache.cache[fingerprint],
+        capture_old_counts,
+    )
+    jax.tree.map(
+        lambda s, u: set_from_dict(s.rng_counters, u),
+        scopes,
+        updated_counts,
+    )
+
+
 def jit(
     fn: Callable[..., Any],
     variables: CollectionFilter = True,
@@ -1612,13 +1682,18 @@ def jit(
       mutable = tuple(_hashable_filter(scope.mutable) for scope in scopes)
 
       rng_groups = jax.tree.map(
-          lambda x: x.fold() if isinstance(x, LazyRng) else x,
+          lambda x: x.clear_suffix() if isinstance(x, LazyRng) else x,
           rng_groups,
           is_leaf=lambda x: isinstance(x, LazyRng),
       )
 
       fingerprint = (mutable, module_hash_key)
-      return jitted(fingerprint, variable_groups, rng_groups, *args, **kwargs)
+      capture_old_counts = jax.tree.map(
+          lambda s: CountsHolder.make(s.rng_counters), scopes
+      )
+      res = jitted(fingerprint, variable_groups, rng_groups, *args, **kwargs)
+      _restore_rng_counters(scopes, fingerprint, capture_old_counts)
+      return res
 
   return pack(
       inner, (variables,), (variables,), (rngs,), name='jit', enable_kwargs=True
@@ -1705,3 +1780,64 @@ def remat_scan(
 def _unzip2(xs):
   ys = tuple(zip(*xs))
   return ys if ys else ((), ())
+
+
+def fold_rngs(
+    fn: Callable[..., Any],
+    variables: CollectionFilter = True,
+    rngs: PRNGSequenceFilter = True,
+) -> Callable[..., Any]:
+  # Close over scope_fn & repack_fn to avoid recompilation
+  # this is impure but we use the fingerprint arg to differentiate between cases
+  # where scope_fn or repack_fn actually produce non-identical results.
+  fold_rngs_context = TransformContext[tuple[Callable, Callable]]()
+
+  @functools.wraps(fn)
+  def wrapped_fold_rngs(fingerprint, variable_groups, rng_groups, *args, **kwargs):
+    scope_fn, repack_fn = fold_rngs_context.get()
+    hash_key = fingerprint[1]
+    # fingerprint is only used to differentiate the cache signature
+    # del fingerprint
+    scope = scope_fn(variable_groups, rng_groups)  # pylint: disable=not-callable
+    y = fn(scope, hash_key, *args, **kwargs)
+    return y, repack_fn(scope)  # pylint: disable=not-callable
+
+  def inner_fold_rngs(
+      scope_fn,
+      repack_fn,
+      variable_groups,
+      rng_groups,
+      module_hash_key,
+      *args,
+      **kwargs,
+  ):
+    with fold_rngs_context.push((scope_fn, repack_fn)):
+      scopes: list[Scope] = jax.tree_util.tree_leaves(
+          scope_fn(variable_groups, rng_groups)
+      )
+      mutable = tuple(_hashable_filter(scope.mutable) for scope in scopes)
+
+      rng_groups = jax.tree.map(
+          lambda x: x.clear_suffix() if isinstance(x, LazyRng) else x,
+          rng_groups,
+          is_leaf=lambda x: isinstance(x, LazyRng),
+      )
+
+      fingerprint = (mutable, module_hash_key)
+      capture_old_counts = jax.tree.map(
+          lambda s: CountsHolder.make(s.rng_counters), scopes
+      )
+      res = wrapped_fold_rngs(
+          fingerprint, variable_groups, rng_groups, *args, **kwargs
+      )
+      _restore_rng_counters(scopes, fingerprint, capture_old_counts)
+      return res
+
+  return pack(
+      inner_fold_rngs,
+      (variables,),
+      (variables,),
+      (rngs,),
+      name='fold_rngs',
+      enable_kwargs=True,
+  )
