@@ -22,6 +22,7 @@ to keep track of how variables should be partitioned with ``jax.pjit``.
 """
 
 import abc
+import dataclasses
 import functools
 from typing import Any, Generic, TypeVar
 from collections.abc import Callable
@@ -58,13 +59,13 @@ class AxisMetadata(Generic[A], metaclass=abc.ABCMeta):
   def unbox(self) -> A:
     """Returns the content of the AxisMetadata box.
 
-    Note that unlike ``meta.unbox`` the unbox call should recursively unbox
+    Note that unlike ``meta.unbox`` the unbox call should not recursively unbox
     metadata. It should simply return value that it wraps directly even
     if that value itself is an instance of AxisMetadata.
 
     In practise, AxisMetadata subclasses should be registered as PyTree nodes to
     support passing instances to JAX and Flax APIs. The leaves returned for this
-    note should correspond to the value returned by unbox.
+    node should correspond to the value returned by unbox.
 
     Returns:
       The unboxed value.
@@ -287,6 +288,19 @@ class Partitioned(struct.PyTreeNode, AxisMetadata[A]):
     """Returns the ``NamedSharding`` for this partitioned value."""
     return jax.sharding.NamedSharding(mesh, self.get_partition_spec())
 
+  def to_nnx_metadata(self) -> dict[str, Any]:
+    """Return a dict of metadata that can translate into an `nnx.Variable`."""
+    metadata = vars(self)
+    metadata['sharding'] = metadata.pop('names')
+    return metadata
+
+  @classmethod
+  def from_nnx_metadata(cls, metadata: dict[str, Any]):
+    """Given a dict of `nnx.Variable` format metadata, create a `nn.Partitioned`."""
+    metadata['names'] = metadata.pop('sharding')
+    fields = {x.name for x in dataclasses.fields(cls)}
+    return cls(**{k: v for k, v in metadata.items() if k in fields})
+
 
 def with_partitioning(
     fn: Callable[..., Any],
@@ -319,26 +333,33 @@ def with_partitioning(
   return wrapper
 
 
+def _get_leaf_pspec(x: Any) -> jax.sharding.PartitionSpec | None:
+  if hasattr(x, 'get_partition_spec'):
+    return x.get_partition_spec()
+  # Unboxed arrays, which should be replicated across all devices
+  elif hasattr(x, 'shape'):
+    return jax.sharding.PartitionSpec()
+  else:
+    return None
+
+
 def get_partition_spec(tree: Any) -> Any:
   """Extracts a PartitionSpec tree from a PyTree containing ``Partitioned`` values."""
-
-  def f(x):
-    if isinstance(x, Partitioned):
-      return x.get_partition_spec()
-    # Unboxed arrays, which should be replicated across all devices
-    elif hasattr(x, 'shape'):
-      return jax.sharding.PartitionSpec()
-    else:
-      return None
-
   return jax.tree_util.tree_map(
-      f, tree, is_leaf=lambda x: isinstance(x, Partitioned)
+      _get_leaf_pspec, tree, is_leaf=lambda x: isinstance(x, AxisMetadata)
   )
 
 
 def get_sharding(tree: Any, mesh: jax.sharding.Mesh) -> Any:
   """Extracts a jax.sharding tree from a PyTree containing ``Partitioned`` values and a mesh."""
-  pspec_tree = get_partition_spec(tree)
+  def f(x: Any) -> jax.sharding.Sharding | None:
+    if hasattr(x, 'get_sharding'):
+      return x.get_sharding(mesh)
+    pspec = _get_leaf_pspec(x)
+    if pspec is None:
+      return None
+    return jax.sharding.NamedSharding(mesh, pspec)
+
   return jax.tree_util.tree_map(
-      lambda x: jax.sharding.NamedSharding(mesh, x), pspec_tree
+      f, tree, is_leaf=lambda x: isinstance(x, AxisMetadata)
   )
