@@ -98,44 +98,3 @@ class PReLU(Module):
     return jnp.where(
         inputs >= 0, inputs, jnp.asarray(negative_slope, inputs.dtype) * inputs
     )
-
-class GeGLU(Module):
-    """Gated Linear Unit with GELU (GeGLU) activation function.
-
-    GeGLU is a Flax layer that combines a linear transformation with a GELU
-    activation function in a gating mechanism. It is often used in Transformer models
-    to provide non-linear capabilities while preserving a strong linear component.
-
-    Example usage::
-        >>> import flax.linen as nn
-
-        >>> class TransformerBlock(nn.Module):
-        ...   @nn.compact
-        ...   def __call__(self, x):
-        ...     x = nn.Dense(2)(x)
-        ...     x = nn.GeGLU()(x) # initialized
-        ...     return x
-
-    Attributes:
-        features: the number of output features (default: None).
-    """
-    output_dim: int = -1
-
-    @compact
-    def __call__(self, inputs: Array) -> Array:
-        """Applies the GeGLU activation to the inputs.
-
-        Args:
-            inputs: the nd-array to apply the GeGLU activation function to.
-
-        Returns:
-            The transformed input.
-        """
-        if self.output_dim == -1:
-          output_dim = inputs.shape[-1]
-        else:
-            output_dim = self.output_dim
-
-        x = Dense(output_dim * 2)(inputs)
-        x, gate = x[..., : output_dim], x[..., output_dim :]
-        return x * gelu(gate)
\ No newline at end of file
